@@ -688,20 +688,23 @@ class PersepolisDB():
         # return the results
         return dictionary
 
-    def insertInVideoFinderTable2(self, dict_):
+    def insertInVideoFinderTable2(self, list_):
         self.lockCursor()
-        self.persepolis_db_cursor.execute("""INSERT INTO video_finder_db_table2 VALUES(NULL,
-                                                                                :gid,
-                                                                                :download_status,
-                                                                                :file_name,
-                                                                                :eta,
-                                                                                :download_speed_str,
-                                                                                :downloaded_size,
-                                                                                :file_size,
-                                                                                :download_percent,
-                                                                                :fragments,
-                                                                                :error_message
-                                                                                )""", dict_)
+
+        for dictionary in list_:
+
+            self.persepolis_db_cursor.execute("""INSERT INTO video_finder_db_table2 VALUES(NULL,
+                                                                                    :gid,
+                                                                                    :download_status,
+                                                                                    :file_name,
+                                                                                    :eta,
+                                                                                    :download_speed_str,
+                                                                                    :downloaded_size,
+                                                                                    :file_size,
+                                                                                    :download_percent,
+                                                                                    :fragments,
+                                                                                    :error_message
+                                                                                    )""", dictionary)
         self.persepolis_db_connection.commit()
         self.lock = False
 
@@ -1243,10 +1246,10 @@ class PersepolisDB():
         # find download items is download_db_table with status = "downloading" or "waiting" or paused or scheduled
         if category:
             self.persepolis_db_cursor.execute("""SELECT gid FROM download_db_table WHERE (category = '{}') AND (status = 'downloading' OR status = 'waiting'
-                                            OR status = 'scheduled' OR status = 'paused')""".format(str(category)))
+                                            OR status = 'scheduled' OR status = 'paused' OR status = 'creating download file')""".format(str(category)))
         else:
             self.persepolis_db_cursor.execute("""SELECT gid FROM download_db_table WHERE (status = 'downloading' OR status = 'waiting'
-                                            OR status = 'scheduled' OR status = 'paused')""")
+                                            OR status = 'scheduled' OR status = 'paused' OR status = 'creating download file')""")
 
         # create a list for returning answer
         result = self.persepolis_db_cursor.fetchall()
@@ -1260,14 +1263,14 @@ class PersepolisDB():
 
         return gid_list
 
-# this method returns items with 'downloading' or 'waiting' status
+# this method returns items with 'downloading' or 'waiting' or 'creating download file' status
     def returnDownloadingItems(self):
         # lock data base
         self.lockCursor()
 
         # find download items is download_db_table with status = "downloading" or "waiting" or paused or scheduled
         self.persepolis_db_cursor.execute(
-            """SELECT gid FROM download_db_table WHERE (status = 'downloading' OR status = 'waiting')""")
+            """SELECT gid FROM download_db_table WHERE (status = 'downloading' OR status = 'waiting' OR status = 'creating download file')""")
 
         # create a list for returning answer
         result = self.persepolis_db_cursor.fetchall()
