@@ -20,10 +20,9 @@ import random
 import threading
 import os
 import errno
-from persepolis.scripts.useful_tools import convertTime, humanReadableSize, freeSpace, headerToDict, readCookieJar, getFileNameFromLink
+from persepolis.scripts.useful_tools import convertTime, humanReadableSize, freeSpace, headerToDict, readCookieJar, getFileNameFromLink, returnNewFileName
 from persepolis.scripts.osCommands import makeDirs, moveFile
 from persepolis.scripts import logger
-from persepolis.scripts.bubble import notifySend
 from persepolis.constants import VERSION
 import json
 from requests.adapters import HTTPAdapter
@@ -40,7 +39,7 @@ class Download():
         self.download_speed_str = "0"
         self.gid = gid
 
-        # download_status can be in waiting, downloading, stop, error, paused
+        # download_status can be in waiting, downloading, stop, error, paused, creating download file
         self.download_status = 'waiting'
 
         self.link = add_link_dictionary['link']
@@ -71,7 +70,13 @@ class Download():
         self.not_converted_download_speed = 0
         self.download_percent = 0
         self.error_message = ''
-        self.close_status = False
+
+        # this flag notify that download finished(stopped, complete or error)
+        # in this situation download status must be written to the database
+        # None means, Download not finished yet.
+        # False meanse, Download has been finished, but download status must be written to the database
+        # True meanse, Download status has been written to the database
+        self.write_it_to_the_database = None
         # check certificate
         if str(main_window.persepolis_setting.value('settings/dont-check-certificate')) == 'yes':
             self.check_certificate = False
@@ -144,8 +149,13 @@ class Download():
     def setRetry(self):
         # set retry numbers.
         # backoff_factor will help to apply delays between attempts to avoid failing again
-        retry = Retry(connect=self.retry, backoff_factor=self.retry_wait)
-        adapter = HTTPAdapter(max_retries=retry)
+        retry_strategy = Retry(
+            total=self.retry,
+            backoff_factor=self.retry_wait,
+            status_forcelist=[429, 500, 502, 503, 504],
+            allowed_methods=["HEAD", "GET", "OPTIONS"]
+        )
+        adapter = HTTPAdapter(max_retries=retry_strategy)
         self.requests_session.mount('http://', adapter)
         self.requests_session.mount('https://', adapter)
 
@@ -248,6 +258,10 @@ class Download():
         self.control_json_file_path = os.path.join(
             self.download_path, control_json_file)
 
+        # If we have diffrent files but with same name, So
+        # this fle must be renamed.
+        rename = False
+
         # create json control file if not created before
         try:
             with open(self.control_json_file_path, 'x') as f:
@@ -303,13 +317,14 @@ class Download():
 
                         # check if the download is duplicated
                         # If download item is duplicated, so resume download
-                        # check ETag
+                        # check ETag. Else rename file
                         if 'ETag' in data_dict:
 
                             if data_dict['ETag'] == self.etag:
                                 self.resume = True
                             else:
                                 self.resume = False
+                                rename = True
 
                         # if ETag is not available, then check file size
                         elif 'file_size' in data_dict:
@@ -318,11 +333,14 @@ class Download():
                                 self.resume = True
                             else:
                                 self.resume = False
+                                rename = True
                         else:
                             self.resume = False
+                            rename = True
 
                     # control file is corrupted.
-                    except Exception:
+                    except Exception as e:
+                        logger.sendToLog(str(e) + ' - GID: ' + self.gid, 'DOWNLOAD ERROR')
                         self.resume = False
 
         if self.resuming_suppurt is False:
@@ -334,6 +352,31 @@ class Download():
         else:
             download_file_existance = False
 
+        # If download file not exists so we can rewrite .persepolis file
+        # Else download file name and json file name must be renamed.
+        if rename and download_file_existance:
+            download_file_existance = False
+            # create new file name
+            self.file_name = returnNewFileName(self.download_path, self.file_name)
+
+            # create new name for json file
+            control_json_file = self.file_name + '.persepolis'
+
+            # create new file path
+            self.file_path = os.path.join(self.download_path, self.file_name)
+
+            # create new control file path
+            self.control_json_file_path = os.path.join(
+                self.download_path, control_json_file)
+
+            # create json control file
+            try:
+                with open(self.control_json_file_path, 'x') as f:
+                    f.write("")
+            except Exception as e:
+                logger.sendToLog(str(e) + ' - GID: ' + self.gid, 'DOWNLOAD ERROR')
+                self.resume = False
+
         if self.resume and not (download_file_existance):
             self.resume = False
             create_download_file = True
@@ -348,9 +391,55 @@ class Download():
 
             # if file_size is specified, create an empty file with file_size
             if self.file_size:
-                fp.write(b'\0' * self.file_size)
+                # check for free space
+                free_space = freeSpace(self.download_path)
 
-            fp.close()
+                if free_space is not None:
+
+                    # compare free disk space and file_size
+                    if free_space >= self.file_size:
+
+                        # sets the size of each chunk to 1MiB
+                        CHUNK_SIZE = (1024 ** 2)
+                        # creates a byte string of zeroes with a size of 1MiB.
+                        # These bytes will be used for writing to the file later.
+                        zero_chunk = b'\0' * CHUNK_SIZE
+                        # This variable indicates how many bytes still need to be written.
+                        remaining = self.file_size
+
+                        # continue the loop until writing ends.
+                        while remaining > 0 and self.download_status != 'stopped':
+                            # determines how much data should be written in this iteration.
+                            # This value can be equal to CHUNK_SIZE,
+                            # but if remaining is less than CHUNK_SIZE, only the remaining amount will be written.
+                            to_write = min(CHUNK_SIZE, remaining)
+
+                            # writes the zero data up to the calculated amount (to_write) to the file.
+                            fp.write(zero_chunk[:to_write])
+                            # updates the remaining amount to indicate how much of the file still needs to be written.
+                            remaining -= to_write
+
+                        if self.download_status != 'stopped':
+                            logger.sendToLog('Empty file has been created!' + ' - GID: ' + self.gid, 'DOWNLOADS')
+                            fp.close()
+                            return True
+                        else:
+                            # Download canceled by user! Delete unfinished empty file.
+                            fp.close()
+                            os.remove(self.file_path)
+                            return False
+                    else:
+                        fp.close()
+                        return False
+                else:
+                    fp.close()
+                    return False
+
+            else:
+                fp.close()
+                return True
+        else:
+            return True
 
     def definePartSizes(self):
         # download_infromation_list contains 64 lists.
@@ -817,6 +906,8 @@ class Download():
         if self.download_status == 'complete':
 
             logger.sendToLog('Download complete. - GID: ' + self.gid, 'DOWNLOADS')
+            self.download_percent = 100
+            self.eta = '0s'
 
         # If the download is not complete and the user has not stopped the download, then the download has encountered an error.
         elif self.download_status != 'stopped':
@@ -868,14 +959,13 @@ class Download():
         sigma_now = self.nowTime()
 
         # while current time is not equal to end_time, continue the loop
-        while sigma_end != sigma_now and (self.download_status == 'downloading' or self.download_status == 'paused'):
-
+        while sigma_end != sigma_now and (self.download_status not in ['stopped', 'error']):
             # get current time
             sigma_now = self.nowTime()
             time.sleep(2.1)
 
         # Time is up!
-        if (self.download_status == 'downloading' or self.download_status == 'paused'):
+        if self.download_status not in ['stopped', 'error']:
             logger.sendToLog("Time is up! - GID:" + self.gid, "DOWNLOADS")
 
             # stop download
@@ -922,14 +1012,13 @@ class Download():
                 self.main_window.persepolis_db.setDefaultGidInAddlinkTable(self.gid, start_time=True)
 
         if self.download_status != 'stopped':
+            self.download_status = 'creating download file'
             # start download
             headers = self.getFileSize()
 
             if headers != {}:
                 self.setRetry()
 
-                self.download_status = 'downloading'
-
                 # if user set end_time
                 if self.end_time:
                     self.runEndTimeThread()
@@ -940,17 +1029,23 @@ class Download():
 
                 self.getFileTag()
 
-                self.createControlFile()
+                enough_free_space = self.createControlFile()
+                if self.download_status != 'stopped':
+                    self.download_status = 'downloading'
 
-                self.definePartSizes()
+                    if enough_free_space:
+                        self.definePartSizes()
 
-                self.runProgressBar()
+                        self.runProgressBar()
 
-                self.runDownloadThreads()
+                        self.runDownloadThreads()
 
-                self.checkDownloadProgress()
+                        self.checkDownloadProgress()
+                    else:
+                        self.download_status = 'error'
 
                 self.close()
+
             else:
                 # headers is missing
                 self.download_status = 'error'
@@ -1006,8 +1101,19 @@ class Download():
         for thread in self.thread_list:
             thread.join()
 
+        self.write_it_to_the_database = False
         logger.sendToLog("persepolis_lib is closed!", 'DOWNLOADS')
-        self.close_status = True
+
+        # remove it from download_sessions_list when download status has been written to the database.
+        for download_session_dict in self.main_window.download_sessions_list:
+            if download_session_dict['gid'] == self.gid:
+
+                # Wait until the information is written to the database.
+                while self.write_it_to_the_database is False:
+                    time.sleep(0.1)
+
+                # remove item
+                self.main_window.download_sessions_list.remove(download_session_dict)
 
     # This method returns download status
     def tellStatus(self):
@@ -1049,50 +1155,16 @@ class Download():
     def downloadCompleteAction(self, new_download_path):
 
         # rename file if file already existed
-        i = 1
+        self.file_name = returnNewFileName(new_download_path, self.file_name)
         new_file_path = os.path.join(new_download_path, self.file_name)
 
-        while os.path.isfile(new_file_path):
-            file_name_split = self.file_name.split('.')
-            extension_length = len(file_name_split[-1]) + 1
+        # move the file to the download folder
+        move_answer = moveFile(str(self.file_path), str(new_file_path), 'file')
 
-            new_name = self.file_name[0:-extension_length] + \
-                '_' + str(i) + self.file_name[-extension_length:]
-            new_file_path = os.path.join(new_download_path, new_name)
-            i = i + 1
-
-        free_space = freeSpace(new_download_path)
-
-        if free_space is not None and self.file_size is not None:
-
-            # compare free disk space and file_size
-            if free_space >= self.file_size:
-
-                # move the file to the download folder
-                move_answer = moveFile(str(self.file_path), str(new_file_path), 'file')
-
-                if not (move_answer):
-                    # write error message in log
-                    logger.sendToLog('Persepolis can not move file' + ' - GID: ' + self.gid, "ERROR")
-                    new_file_path = self.file_path
-
-            else:
-                # notify user if we have insufficient disk space
-                # and do not move file from temp download folder to download folder
-                new_file_path = self.file_path
-                logger.sendToLog('Insufficient disk space in download folder' + ' - GID: ' + self.gid, "ERROR")
-
-                # show notification
-                notifySend("Insufficient disk space!", 'Please change download folder',
-                           10000, 'fail', parent=self.main_window)
-
-        else:
-            # move the file to the download folder
-            move_answer = moveFile(str(self.file_path), str(new_file_path), 'file')
-
-            if not (move_answer):
-                logger.sendToLog('Persepolis can not move file' + ' - GID: ' + self.gid, "ERROR")
-                new_file_path = self.file_path
+        if not (move_answer):
+            # write error message in log
+            logger.sendToLog('Persepolis can not move file' + ' - GID: ' + self.gid, "ERROR")
+            new_file_path = self.file_path
 
         return str(new_file_path)
 
