@@ -15,29 +15,32 @@
 """
 try:
     from PySide6.QtWidgets import QCheckBox, QPushButton, QTextEdit, QFrame, QLabel, QComboBox, QHBoxLayout, QApplication
-    from PySide6.QtCore import QThread, Signal, QCoreApplication, QTranslator, QLocale
+    from PySide6.QtCore import QThread, Signal, QCoreApplication, QTranslator, QLocale, QPoint, QSize
+    from PySide6.QtGui import QPixmap
 except:
     from PyQt5.QtWidgets import QCheckBox, QPushButton, QTextEdit, QFrame, QLabel, QComboBox, QHBoxLayout, QApplication
-    from PyQt5.QtCore import QThread, QCoreApplication, QTranslator, QLocale
+    from PyQt5.QtCore import QThread, QCoreApplication, QTranslator, QLocale, QPoint, QSize
+    from PyQt5.QtGui import QPixmap
     from PyQt5.QtCore import pyqtSignal as Signal
 
 from persepolis.scripts.useful_tools import determineConfigFolder, dictToHeader
 from persepolis.scripts.addlink import AddLinkWindow
 from persepolis.scripts import logger, osCommands
 from persepolis.scripts.spider import spider
-from persepolis.constants import VERSION
+from persepolis.constants import VERSION, OS
 from functools import partial
 from time import time
-from random import random
+from random import random, randint
 from copy import deepcopy
 import yt_dlp as youtube_dl
 import urllib
 import re
 import os
+import glob
+import platform
 
 # write youtube_dl version in log
-logger.sendToLog('yt-dlp version: '
-                 + str(youtube_dl.version.__version__),
+logger.sendToLog('yt-dlp version: ' + str(youtube_dl.version.__version__),
                  'INITIALIZATION')
 
 # download manager config folder .
@@ -46,11 +49,22 @@ config_folder = determineConfigFolder()
 # persepolis tmp folder path
 persepolis_tmp = os.path.join(config_folder, 'persepolis_tmp')
 
+os_type = platform.system()
+home_address = os.path.expanduser("~")
+
+if os_type is OS.WINDOWS:
+    tmp_folder_system = os.path.join(home_address, "AppData", "Local", "Temp")
+    # make tmp folder if not exists
+    osCommands.makeDirs(tmp_folder_system)
+else:
+    tmp_folder_system = '/tmp'
+
 
 class MediaListFetcherThread(QThread):
     RESULT = Signal(dict)
     cookies = '# HTTP cookie file.\n'  # We shall write it in a file when thread starts.
     LOADCOOKIEFILESIGNAL = Signal(str)
+    THUMBNAILSIGNAL = Signal(str)
 
     def __init__(self, receiver_slot, video_dict, main_window):
         super().__init__()
@@ -72,21 +86,36 @@ class MediaListFetcherThread(QThread):
                                         'no_warnings': True,
                                         'no-check-certificates': self.dont_check_certificate
                                         }
+        # create random name for thumbnail
+        self.thumbnail_file_name = str(randint(0, 1500))
+
+        # file must be downloaded in temp folder of system
+        self.thumbnail_path = os.path.join(tmp_folder_system, self.thumbnail_file_name)
+        self.youtube_dl_options_dict_thumbnails = {'skip_download': True,
+                                                   'quiet': True,
+                                                   'writethumbnail': True,
+                                                   'outtmpl': self.thumbnail_path,
+                                                   }
 
         # cookies
         self.youtube_dl_options_dict['cookies'] = str(self.cookie_path)
+        self.youtube_dl_options_dict_thumbnails['cookies'] = str(self.cookie_path)
 
         # referer
         if 'referer' in video_dict.keys() and video_dict['referer']:
             self.youtube_dl_options_dict['referer'] = str(video_dict['referer'])
+            self.youtube_dl_options_dict_thumbnails['referer'] = str(video_dict['referer'])
 
         # user_agent
         if 'user_agent' in video_dict.keys() and video_dict['user_agent']:
             self.youtube_dl_options_dict['user-agent'] = str(video_dict['user_agent'])
+            self.youtube_dl_options_dict_thumbnails['user-agent'] = str(video_dict['user_agent'])
+
         else:
             # set PersepolisDM user agent
             video_dict['user_agent'] = 'PersepolisDM/' + str(VERSION.version_str)
             self.youtube_dl_options_dict['user-agent'] = 'PersepolisDM/' + str(VERSION.version_str)
+            self.youtube_dl_options_dict_thumbnails['user-agent'] = 'PersepolisDM/' + str(VERSION.version_str)
 
         # load_cookies
         if 'load_cookies' in video_dict.keys() and video_dict['load_cookies']:
@@ -100,17 +129,21 @@ class MediaListFetcherThread(QThread):
             ip_port = '{}:{}'.format(video_dict['ip'], str(video_dict['port']))
 
             if video_dict['proxy_user']:
-                proxy_argument = '{}://{}:{}@{}'.format(video_dict['proxy_type'], video_dict['proxy_user'], video_dict['proxy_passwd'], ip_port)
+                proxy_argument = '{}://{}:{}@{}'.format(video_dict['proxy_type'],
+                                                        video_dict['proxy_user'], video_dict['proxy_passwd'], ip_port)
 
             else:
                 proxy_argument = '{}://{}'.format(video_dict['proxy_type'], ip_port)
 
             self.youtube_dl_options_dict['proxy'] = str(proxy_argument)
+            self.youtube_dl_options_dict_thumbnails['proxy'] = str(proxy_argument)
 
         if video_dict['download_user']:
 
             self.youtube_dl_options_dict['username'] = str(video_dict['download_user'])
             self.youtube_dl_options_dict['password'] = str(video_dict['download_passwd'])
+            self.youtube_dl_options_dict_thumbnails['username'] = str(video_dict['download_user'])
+            self.youtube_dl_options_dict_thumbnails['password'] = str(video_dict['download_passwd'])
 
         if video_dict['link']:
             self.youtube_link = str(video_dict['link'])
@@ -150,6 +183,26 @@ class MediaListFetcherThread(QThread):
             except Exception as ex:
                 logger.sendToLog(ex, "DOWNLOAD ERROR")
 
+        try:  # get video thumbnail
+            ydl_thumbnail = youtube_dl.YoutubeDL(self.youtube_dl_options_dict_thumbnails)
+            with ydl_thumbnail:
+                result = ydl_thumbnail.download(
+                    self.youtube_link
+                )
+
+            # find thumbnail name with file extension
+            # we know the file name but we don't know file extension.
+            # create pattern
+            file_pattern = self.thumbnail_path + '*'
+            # search for pattern and return first item in the list
+            files_list = glob.glob(file_pattern)
+
+            self.THUMBNAILSIGNAL.emit(files_list[0])
+
+        except Exception as ex:
+            ex = 'Thumbnail error:' + str(ex)
+            logger.sendToLog(ex, "DOWNLOAD ERROR")
+
         self.LOADCOOKIEFILESIGNAL.emit(self.cookie_path)
         self.RESULT.emit(ret_val)
 
@@ -201,6 +254,7 @@ class VideoFinderAddLink(AddLinkWindow):
         self.no_video_list = []
         self.video_audio_list = []
         self.cookie_path = None
+        self.thumbnail_path = ''
 
         self.media_title = ''
 
@@ -268,6 +322,10 @@ class VideoFinderAddLink(AddLinkWindow):
                        self.audio_format_selection_comboBox]:
             advanced_format_selection_horizontalLayout.addWidget(widget)
 
+        # thumbnail picture
+        self.thumbnail_label = QLabel(self.link_frame)
+        self.change_name_horizontalLayout.addWidget(self.thumbnail_label)
+
         # Set Texts
         self.url_submit_pushButtontton.setText(QCoreApplication.translate("ytaddlink_src_ui_tr", 'Fetch Media List'))
         self.select_format_label.setText(QCoreApplication.translate("ytaddlink_src_ui_tr", 'Select a format'))
@@ -303,6 +361,13 @@ class VideoFinderAddLink(AddLinkWindow):
         self.link_lineEdit.textChanged.connect(self.linkLineChangedHere)
 
         self.setMinimumSize(650, 480)
+        # set window size and position
+        size = self.persepolis_setting.value(
+            'VideoFinderAddLinkWindow/size', QSize(652, 480))
+        position = self.persepolis_setting.value(
+            'VideoFinderAddLinkWindow/position', QPoint(300, 300))
+        self.resize(size)
+        self.move(position)
 
         self.status_box_textEdit.hide()
         self.format_selection_frame.hide()
@@ -395,6 +460,16 @@ class VideoFinderAddLink(AddLinkWindow):
         self.parent.threadPool.append(fetcher_thread)
         self.parent.threadPool[-1].start()
         self.parent.threadPool[-1].LOADCOOKIEFILESIGNAL.connect(self.setLoadCookie)
+        self.parent.threadPool[-1].THUMBNAILSIGNAL.connect(self.showThumbnail)
+
+    # Thumbnail is available! So show it!
+    def showThumbnail(self, thumbnail_path):
+        self.thumbnail_path = thumbnail_path
+        pixmap = QPixmap(self.thumbnail_path)
+
+        # Thumbnail height must be 1/5 height of window.
+        height_of_window = self.height()
+        self.thumbnail_label.setPixmap(pixmap.scaledToHeight(height_of_window // 5))
 
     def setLoadCookie(self, str):
         if os.path.isfile(str):
@@ -428,8 +503,7 @@ class VideoFinderAddLink(AddLinkWindow):
 
                     if self.audio_format_selection_comboBox.currentText() != 'No audio':
                         self.change_name_lineEdit.setText(self.media_title)
-                        self.extension_label.setText('.'
-                                                     + self.no_video_list[int(self.audio_format_selection_comboBox.currentIndex()) - 1]['ext'])
+                        self.extension_label.setText('.' + self.no_video_list[int(self.audio_format_selection_comboBox.currentIndex()) - 1]['ext'])
 
                         self.change_name_checkBox.setChecked(True)
                     else:
@@ -438,15 +512,13 @@ class VideoFinderAddLink(AddLinkWindow):
             elif combobox == 'audio':
                 if self.audio_format_selection_comboBox.currentText() != 'No audio' and self.video_format_selection_comboBox.currentText() == 'No video':
                     self.change_name_lineEdit.setText(self.media_title)
-                    self.extension_label.setText('.'
-                                                 + self.no_video_list[index - 1]['ext'])
+                    self.extension_label.setText('.' + self.no_video_list[index - 1]['ext'])
 
                     self.change_name_checkBox.setChecked(True)
 
                 elif (self.audio_format_selection_comboBox.currentText() == 'No audio' and self.video_format_selection_comboBox.currentText() != 'No video') or (self.audio_format_selection_comboBox.currentText() != 'No audio' and self.video_format_selection_comboBox.currentText() != 'No video'):
                     self.change_name_lineEdit.setText(self.media_title)
-                    self.extension_label.setText('.'
-                                                 + self.no_audio_list[int(self.video_format_selection_comboBox.currentIndex()) - 1]['ext'])
+                    self.extension_label.setText('.' + self.no_audio_list[int(self.video_format_selection_comboBox.currentIndex()) - 1]['ext'])
 
                     self.change_name_checkBox.setChecked(True)
 
@@ -462,13 +534,15 @@ class VideoFinderAddLink(AddLinkWindow):
         root, ext = os.path.splitext(parsed.path)
         return ext[1:]
 
-    def fetchedResult(self, media_dict): # noqa
+    def fetchedResult(self, media_dict):
 
-        self.url_submit_pushButtontton.setEnabled(True)
         if 'error' in media_dict.keys():
 
             self.status_box_textEdit.setText('<font color="#f11">' + str(media_dict['error']) + '</font>')
             self.status_box_textEdit.show()
+
+            # enable fetch media button again
+            self.url_submit_pushButtontton.setEnabled(True)
         else:  # Show the media list
 
             # add no audio and no video options to the comboboxes
@@ -491,7 +565,10 @@ class VideoFinderAddLink(AddLinkWindow):
 
             try:
                 i = 0
+                media_dict_lenght = len(media_dict['formats'])
                 for f in media_dict['formats']:
+
+                    url_ext = self.getFileExtension(f['url'])
                     no_audio = False
                     no_video = False
                     text = ''
@@ -548,6 +625,9 @@ class VideoFinderAddLink(AddLinkWindow):
                         for key in more_options.keys():
                             input_dict[key] = more_options[key]
 
+                    if url_ext == 'm3u8':
+                        text = text + ' m3u8'
+
                     # Add current format to the related comboboxes
                     if no_audio:
                         combobox_type = 'video'
@@ -562,12 +642,15 @@ class VideoFinderAddLink(AddLinkWindow):
                         index = self.audio_format_selection_comboBox.count() - 1
 
                     else:
+                        if i == 0 and (media_dict_lenght > 1):
+                            text = 'Worst quality: ' + text
+                        elif i == (media_dict_lenght - 1):
+                            text = 'Best quality: ' + text
                         combobox_type = 'media'
                         self.video_audio_list.append(f)
                         self.media_comboBox.addItem(text)
                         index = self.media_comboBox.count() - 1
 
-                    url_ext = self.getFileExtension(f['url'])
                     # we can't get size of file from m3u8 format.
                     if not (size_available) and url_ext != 'm3u8':
                         size_fetcher = FileSizeFetcherThread(input_dict, text, combobox_type, index)
@@ -649,6 +732,7 @@ class VideoFinderAddLink(AddLinkWindow):
                     self.video_format_selection_comboBox.setItemText(index, '{} - {}'.format(text, result['file_size']))
                 else:
                     self.media_comboBox.setItemText(index, '{} - {}'.format(text, result['file_size']))
+                    self.video_audio_list[index]['file_size'] = result['file_size']
 
         except Exception as ex:
             logger.sendToLog(ex, "DOWNLOAD ERROR")
@@ -659,6 +743,13 @@ class VideoFinderAddLink(AddLinkWindow):
         else:
             self.url_submit_pushButtontton.setEnabled(True)
 
+    def closeEvent(self, event):
+        self.persepolis_setting.setValue('VideoFinderAddLinkWindow/size', self.size())
+        self.persepolis_setting.setValue('VideoFinderAddLinkWindow/position', self.pos())
+        self.persepolis_setting.sync()
+        osCommands.remove(self.thumbnail_path)
+        event.accept()
+
     # This method collects additional information like proxy ip, user, password etc.
     def collectMoreOptions(self):
         options = {'ip': None, 'port': None, 'proxy_user': None, 'proxy_passwd': None, 'download_user': None,
@@ -703,7 +794,7 @@ class VideoFinderAddLink(AddLinkWindow):
 
     # user submitted information by pressing ok_pushButton, so get information
     # from VideoFinderAddLink window and return them to the mainwindow with callback!
-    def okButtonPressed(self, download_later, button=None): # noqa
+    def okButtonPressed(self, download_later, button=None):  # noqa
 
         link_list = []
         # separate audio format and video format is selected.
@@ -856,7 +947,7 @@ class VideoFinderAddLink(AddLinkWindow):
         if self.referer_lineEdit.text() != '':
             referer = self.referer_lineEdit.text()
         else:
-            referer = None
+            referer = self.link_lineEdit.text()
 
         # header
         if self.header_lineEdit.text() != '':
