@@ -18,11 +18,11 @@ import time
 import threading
 from persepolis.constants import VERSION
 from persepolis.scripts import logger
-from persepolis.scripts.useful_tools import humanReadableSize, convertTime
+from persepolis.scripts.useful_tools import humanReadableSize, convertTime, returnNewFileName
+from persepolis.scripts.osCommands import makeDirs, moveFile
 from urllib.parse import urlparse, unquote
 from pathlib import Path
 import os
-import multiprocessing
 
 
 # This class gets yt-dlp log messages.
@@ -49,7 +49,7 @@ class Ytdp_Logger():
 
 # This class downloads m3u8 format files.
 class Ytdp_Download():
-    def __init__(self, add_link_dictionary, main_window, gid):
+    def __init__(self, add_link_dictionary, main_window, gid, single_video_link=False):
         self.downloaded_size = 0
         self.finished_threads = 0
         self.eta = "0"
@@ -86,7 +86,16 @@ class Ytdp_Download():
         self.not_converted_download_speed = 0
         self.download_percent = 0
         self.error_message = ''
-        self.close_status = False
+        self.single_video_link = single_video_link
+        self.yt_dlp_exited = False
+
+        # this flag notify that download finished(stopped, complete or error)
+        # in this situation download status must be written to the database
+        # None means, Download not finished yet.
+        # False meanse, Download has been finished, but download status must be written to the database
+        # True meanse, Download status has been written to the database
+        self.write_it_to_the_database = None
+
         # check certificate
         if str(main_window.persepolis_setting.value('settings/dont-check-certificate')) == 'yes':
             self.dont_check_certificate = True
@@ -245,6 +254,12 @@ class Ytdp_Download():
         self.thread_list.append(end_time_thread)
 
     def getStatus(self, data):
+        # Download stopped by user!
+        # raise and exception for stopping download!
+        if self.download_status == 'stopped':
+            raise Exception('Download stopped')
+            return
+
         if 'filename' in data.keys():
             download_path_pluse_name = data['filename']
             self.file_name = Path(download_path_pluse_name).name
@@ -256,8 +271,7 @@ class Ytdp_Download():
         if 'speed' in data.keys():
             if data['speed']:
                 download_speed, speed_unit = humanReadableSize(float(data['speed']), 'speed')
-                self.download_speed_str = (str(download_speed)
-                                           + " " + speed_unit + "/s")
+                self.download_speed_str = (str(download_speed) + " " + speed_unit + "/s")
 
         if 'downloaded_bytes' in data.keys():
             if data['downloaded_bytes']:
@@ -386,15 +400,25 @@ class Ytdp_Download():
         try:
             self.ytdl_session.download([self.link])
         except Exception as e:
-            self.error_message = str(e)
-            self.download_status = 'error'
-            # update data_base
-            dict_ = {'gid': self.gid,
-                     'download_status': self.download_status,
-                     'error_message': self.error_message}
-            self.main_window.persepolis_db.updateVideoFinderTable2(dict_)
+            if self.download_status != 'stopped':
+                # So download didn't stop by user!
+                self.error_message = str(e)
+                self.download_status = 'error'
+                # update data_base
+                dict_ = {'gid': self.gid,
+                         'download_status': self.download_status,
+                         'error_message': self.error_message}
+                self.main_window.persepolis_db.updateVideoFinderTable2(dict_)
+
+        self.yt_dlp_exited = True
 
     def start(self):
+        # Create download_path if not existed
+        try:
+            makeDirs(self.download_path)
+        except:
+            pass
+
         self.createSession()
         # update status and last_try_date in data_base
         if self.start_time:
@@ -436,8 +460,11 @@ class Ytdp_Download():
             if self.end_time:
                 self.runEndTimeThread()
 
-            self.process = multiprocessing.Process(target=self.download)
-            self.process.start()
+            # Start the download thread
+            download_thread = threading.Thread(target=self.download)
+            download_thread.setDaemon(True)
+            download_thread.start()
+
             self.checkDownloadProgress()
             self.close()
         else:
@@ -447,18 +474,65 @@ class Ytdp_Download():
     def close(self):
         # if download complete, so delete control file
         if self.download_status == 'complete':
+            # if user specified download_path is equal to persepolis_setting download_path,
+            # then subfolder must added to download path.
+            if self.main_window.persepolis_setting.value('settings/download_path') == self.download_path and self.single_video_link:
+
+                # return new download_path
+                if self.main_window.persepolis_setting.value('settings/subfolder') == 'yes':
+                    new_download_path = os.path.join(self.download_path, 'Videos')
+
+                    file_path = self.downloadCompleteAction(new_download_path)
+            else:
+                # keep user specified download_path
+                file_path = self.file_path
+
             # update download_path in addlink_db_table
             add_link_dictionary = self.main_window.persepolis_db.searchGidInAddLinkTable(self.gid)
 
-            add_link_dictionary['download_path'] = self.file_path
+            add_link_dictionary['download_path'] = file_path
             self.main_window.persepolis_db.updateAddLinkTable([add_link_dictionary])
 
         # ask threads for exiting.
         for thread in self.thread_list:
             thread.join()
 
+        self.write_it_to_the_database = False
         logger.sendToLog("ytdlp_downloader is closed!", 'DOWNLOADS')
-        self.close_status = True
+
+        # remove it from download_sessions_list when download status has been written to the database.
+        for download_session_dict in self.main_window.download_sessions_list:
+            if download_session_dict['gid'] == self.gid:
+
+                # Wait until the information is written to the database.
+                while self.write_it_to_the_database is False:
+                    time.sleep(0.1)
+
+                # remove item
+                self.main_window.download_sessions_list.remove(download_session_dict)
+
+        # remove gid from single_video_link_gid_list
+        if self.gid in self.main_window.single_video_link_gid_list:
+            self.main_window.single_video_link_gid_list.remove(self.gid)
+
+    def downloadCompleteAction(self, new_download_path):
+
+        # rename file if file already existed
+        self.file_name = returnNewFileName(new_download_path, self.file_name)
+        new_file_path = os.path.join(new_download_path, self.file_name)
+
+        # move the file to the download folder but first make sure yt-dlp has finished.
+        while not (self.yt_dlp_exited):
+            time.sleep(0.1)
+
+        move_answer = moveFile(str(self.file_path), str(new_file_path), 'file')
+
+        if not (move_answer):
+            # write error message in log
+            logger.sendToLog('Persepolis can not move file' + ' - GID: ' + self.gid, "ERROR")
+            new_file_path = self.file_path
+
+        return str(new_file_path)
 
     def downloadPause(self):
         pass
@@ -467,13 +541,13 @@ class Ytdp_Download():
         pass
 
     def downloadStop(self):
-        if self.process:
-            self.process.terminate()
-            self.download_status = 'stopped'
-            # update data_base
-            dict_ = {'gid': self.gid,
-                     'download_status': self.download_status}
-            self.main_window.persepolis_db.updateVideoFinderTable2(dict_)
+        # Change download status to stopped and infrom self.getStatus method
+        self.download_status = 'stopped'
+
+        # update data_base
+        dict_ = {'gid': self.gid,
+                 'download_status': self.download_status}
+        self.main_window.persepolis_db.updateVideoFinderTable2(dict_)
 
     # This method limits download speed
     def limitSpeed(self, limit_value):
